package com.gic.store.service.outer;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.constant.StoreGroupErrorEnum;
import com.gic.store.dao.mapper.TabStoreGroupMapper;
import com.gic.store.dto.StoreGroupDTO;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.exception.StoreGroupException;
import com.gic.store.service.StoreGroupApiService;
import com.gic.store.service.StoreGroupService;
import com.gic.store.service.StoreService;
import com.gic.store.utils.ErrorCode;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Stack;

/**
 * @author guojx
 * @date 2019/6/26 2:24 PM
 */
@Service("storeGroupApiService")
public class StoreGroupApiServiceImpl implements StoreGroupApiService{

    private static final Logger logger = LoggerFactory.getLogger(StoreGroupApiServiceImpl.class);

    private static final int SUCCESS = 1;

    @Autowired
    private TabStoreGroupMapper tabStoreGroupMapper;

    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreGroupService storeGroupService;

    @Override
    public int save(StoreGroupDTO storeGroupDTO) {
        if (tabStoreGroupMapper.isRepeatName(storeGroupDTO.getStoreGroupName(), null, storeGroupDTO.getEnterpriseId()) > 0) {
            throw new StoreGroupException(StoreGroupErrorEnum.Error.getCode(), "名称不能重复");
        }
        int groupLevel = storeGroupDTO.getGroupLevel().intValue();
        if (groupLevel < StoreGroupConstant.ALL_STORE_LEVEL) {
            throw new StoreGroupException(StoreGroupErrorEnum.GroupLevelError.getCode(), StoreGroupErrorEnum.GroupLevelError.getMessage());
        }
        if (groupLevel > StoreGroupConstant.LAST_STORE_GROUP_LEVEL) {
            throw new StoreGroupException(StoreGroupErrorEnum.GroupLevelError.getCode(), StoreGroupErrorEnum.GroupLevelError.getMessage());
        }
        //所有门店的父级ID为0
        if (groupLevel == StoreGroupConstant.ALL_STORE_LEVEL) {
            TabStoreGroup allStoreGroup = tabStoreGroupMapper.selectAllStoreGroup(storeGroupDTO.getEnterpriseId(), StoreGroupConstant.ALL_STORE_LEVEL);
            if (allStoreGroup != null) {
                throw new StoreGroupException(StoreGroupErrorEnum.AllStoreGroupError.getCode(), StoreGroupErrorEnum.AllStoreGroupError.getMessage());
            }
            storeGroupDTO.setParentStoreGroupId(StoreGroupConstant.ALL_STORE_LEVEL);
        } else if (groupLevel == StoreGroupConstant.FIRST_STORE_GROUP_LEVEL) {
            //如果是第一级，则父级ID取所有门店，所有门店在初始化品牌的时候创建
            TabStoreGroup allStoreGroup = tabStoreGroupMapper.selectAllStoreGroup(storeGroupDTO.getEnterpriseId(), StoreGroupConstant.ALL_STORE_LEVEL);
            storeGroupDTO.setParentStoreGroupId(allStoreGroup.getStoreGroupId());
        } else {
            if (storeGroupDTO.getParentStoreGroupId() == null) {
                throw new StoreGroupException(StoreGroupErrorEnum.ParentStoreGroupParamError.getCode(), StoreGroupErrorEnum.ParentStoreGroupParamError.getMessage());
            }
            validIsDefault(storeGroupDTO.getParentStoreGroupId());
        }

        storeGroupDTO.setCreateTime(new Date());
        storeGroupDTO.setUpdateTime(new Date());
        storeGroupDTO.setStatus(1);
        if (storeGroupDTO.getIsDefault() == null) {
            storeGroupDTO.setIsDefault(0);
        }
        //排序值
        storeGroupDTO.setSort(selectStoreGroupMaxSort(storeGroupDTO.getEnterpriseId(), storeGroupDTO.getGroupLevel()) + 1);
        TabStoreGroup record = EntityUtil.changeEntityNew(TabStoreGroup.class, storeGroupDTO);
        tabStoreGroupMapper.insert(record);
        int storeGroupId = record.getStoreGroupId();

                //update 分组链
        TabStoreGroup storeGroupChain = new TabStoreGroup();
        storeGroupChain.setUpdateTime(new Date());
        storeGroupChain.setStoreGroupId(storeGroupId);
        storeGroupChain.setStoreGroupChain(getStoreGroupChain(storeGroupId));
        tabStoreGroupMapper.updateByPrimaryKeySelective(storeGroupChain);

        if (isNeedTransferStore(storeGroupDTO)) {
            //转移门店
            storeService.updateGroupId(storeGroupDTO.getParentStoreGroupId(), storeGroupId);
        }

        if (groupLevel == StoreGroupConstant.ALL_STORE_LEVEL) {
            //初始化品牌，新增所有门店分组之后，新增一条名称叫做 未分组门店 的默认分组
            StoreGroupDTO noStoreGroup = new StoreGroupDTO();
            noStoreGroup.setEnterpriseId(storeGroupDTO.getEnterpriseId());
            noStoreGroup.setGroupLevel(StoreGroupConstant.FIRST_STORE_GROUP_LEVEL);
            noStoreGroup.setStoreGroupName("未分组门店");
            noStoreGroup.setIsDefault(1);
            save(noStoreGroup);
        }
        return storeGroupId;
    }

    @Override
    public int update(StoreGroupDTO storeGroupDTO) {
        if (storeGroupDTO.getStoreGroupId() == null) {
            throw new StoreGroupException(StoreGroupErrorEnum.ParamError.getCode(), "主键不能为空");
        }
        int storeGroupId = storeGroupDTO.getStoreGroupId();
        TabStoreGroup oldStoreGroup = validIsDefault(storeGroupId);
        if (oldStoreGroup.getGroupLevel() != StoreGroupConstant.FIRST_STORE_GROUP_LEVEL) {
            if (storeGroupDTO.getParentStoreGroupId() == null) {
                throw new StoreGroupException(StoreGroupErrorEnum.ParentStoreGroupParamError.getCode(), StoreGroupErrorEnum.ParentStoreGroupParamError.getMessage());
            }
            int storeGroupParentId = storeGroupDTO.getParentStoreGroupId();
            int oldStoreGroupParentId = oldStoreGroup.getParentStoreGroupId();
            boolean hasEditParentId = storeGroupParentId != oldStoreGroupParentId;
            if (hasEditParentId) {
                //验证父级ID的层级和当前层级是否相差一，如果是，则正确，否则，数据异常
                TabStoreGroup parentStoreGroup = tabStoreGroupMapper.selectByPrimaryKey(storeGroupParentId);
                if (parentStoreGroup == null) {
                    throw new StoreGroupException(StoreGroupErrorEnum.ParentStoreGroupParamError.getCode(), "父级ID数据错误，查询不到数据");
                }
                if (oldStoreGroup.getGroupLevel() - parentStoreGroup.getGroupLevel() != 1) {
                    throw new StoreGroupException(StoreGroupErrorEnum.ParentStoreGroupParamError.getCode(), "父级ID数据错误，不是当前的上级层级");
                }

                //如果修改了上级分组，则该分组下的所有子级分组都需要修改分组链数据
                tabStoreGroupMapper.updateStoreGroupChainByStoreGroupId(storeGroupId, oldStoreGroupParentId, storeGroupParentId);

                //修改父级ID
                TabStoreGroup tabStoreGroup =  new TabStoreGroup();
                tabStoreGroup.setStoreGroupId(storeGroupDTO.getStoreGroupId());
                tabStoreGroup.setParentStoreGroupId(storeGroupParentId);
                tabStoreGroup.setUpdateTime(new Date());
                tabStoreGroupMapper.updateByPrimaryKeySelective(tabStoreGroup);

                //如果上级分组有门店，则新建一个未分组数据，然后把所有门店放进去
                if (isNeedTransferStore(storeGroupDTO)) {
                    StoreGroupDTO noStoreGroup = new StoreGroupDTO();
                    noStoreGroup.setEnterpriseId(storeGroupDTO.getEnterpriseId());
                    noStoreGroup.setGroupLevel(oldStoreGroup.getGroupLevel());
                    noStoreGroup.setStoreGroupName("未分组门店" + System.currentTimeMillis());
                    noStoreGroup.setParentStoreGroupId(storeGroupParentId);
                    int noStoreGroupId = save(noStoreGroup);
                    //转移门店
                    storeService.updateGroupId(storeGroupParentId, noStoreGroupId);
                }
            }
        } else {
            throw new StoreGroupException(StoreGroupErrorEnum.ParentStoreGroupParamError.getCode(), "一级分组不需要修改上级分组");
        }
        return SUCCESS;
    }

    @Override
    public int updateStoreGroupName(StoreGroupDTO storeGroupDTO) {
        if (storeGroupDTO.getStoreGroupId() == null) {
            throw new StoreGroupException(StoreGroupErrorEnum.ParamError.getCode(), "主键不能为空");
        }
        if (StringUtils.isBlank(storeGroupDTO.getStoreGroupName())) {
            throw new StoreGroupException(StoreGroupErrorEnum.ParamError.getCode(), "名称不能为空");
        }
        validIsDefault(storeGroupDTO.getStoreGroupId());
        if (tabStoreGroupMapper.isRepeatName(storeGroupDTO.getStoreGroupName(), storeGroupDTO.getStoreGroupId(), storeGroupDTO.getEnterpriseId()) > 0) {
            throw new StoreGroupException(StoreGroupErrorEnum.Error.getCode(), "名称不能重复");
        }
        //修改名称
        TabStoreGroup tabStoreGroup =  new TabStoreGroup();
        tabStoreGroup.setStoreGroupName(storeGroupDTO.getStoreGroupName());
        tabStoreGroup.setStoreGroupId(storeGroupDTO.getStoreGroupId());
        tabStoreGroup.setUpdateTime(new Date());
        tabStoreGroupMapper.updateByPrimaryKeySelective(tabStoreGroup);
        return SUCCESS;
    }

    @Override
    public int sort(StoreGroupDTO storeGroupDTO, boolean isUp) {
        TabStoreGroup tabStoreGroup = tabStoreGroupMapper.selectByPrimaryKey(storeGroupDTO.getStoreGroupId());
        int groupLevel = tabStoreGroup.getGroupLevel();
        int enterpriseId = tabStoreGroup.getEnterpriseId();
        int sort = tabStoreGroup.getSort();

        TabStoreGroup needUpdateSortStoreGroup;
        if (isUp) {
            needUpdateSortStoreGroup = tabStoreGroupMapper.getStoreGroupWhenMaxSort(enterpriseId, groupLevel, sort);
            if (needUpdateSortStoreGroup == null) {
                throw new StoreGroupException(StoreGroupErrorEnum.SortError.getCode(), "已经是第一条，不能上移");
            }
        } else {
            needUpdateSortStoreGroup = tabStoreGroupMapper.getStoreGroupWhenMinSort(enterpriseId, groupLevel, sort);
            if (needUpdateSortStoreGroup == null) {
                throw new StoreGroupException(StoreGroupErrorEnum.SortError.getCode(), "已经是最后一条，不能下移");
            }
        }
        updateSort(needUpdateSortStoreGroup.getStoreGroupId(), sort);
        updateSort(storeGroupDTO.getStoreGroupId(), needUpdateSortStoreGroup.getSort());
        return SUCCESS;
    }

    @Override
    public List<StoreGroupDTO> listStoreGroup(StoreGroupDTO storeGroupDTO) {
        List<TabStoreGroup> tabStoreGroupList = new ArrayList<>(16);
        if (storeGroupDTO.getStoreGroupId() != null) {
            //如果有ID，则查询出该分组以及子集分组数据
            tabStoreGroupList.addAll(listStoreGroupAndChildren(storeGroupDTO.getEnterpriseId(), storeGroupDTO.getStoreGroupId()));

            //获取父级数据
            TabStoreGroup tabStoreGroup = tabStoreGroupMapper.selectByPrimaryKey(storeGroupDTO.getStoreGroupId());
            Stack<TabStoreGroup> parentStoreGroupList = new Stack<>();
            listParentStoreGroup(tabStoreGroup.getParentStoreGroupId(), parentStoreGroupList);
            while (!parentStoreGroupList.empty()) {
                tabStoreGroupList.add(parentStoreGroupList.pop());
            }
        }

        if (storeGroupDTO.getGroupLevel() != null) {
            tabStoreGroupList.addAll(listStoreGroupByLevelAndName(storeGroupDTO.getGroupLevel(), storeGroupDTO.getEnterpriseId(), storeGroupDTO.getStoreGroupName()));
        } else {
            //查询全部
            TabStoreGroup selectAll = new TabStoreGroup();
            selectAll.setEnterpriseId(storeGroupDTO.getEnterpriseId());
            tabStoreGroupList.addAll(tabStoreGroupMapper.listStoreGroup(selectAll));
        }

        return EntityUtil.changeEntityListNew(StoreGroupDTO.class, tabStoreGroupList);
    }

    @Override
    public int remove(int storeGroupId) {
        if (isContainStore(storeGroupId)) {
            throw new StoreGroupException(StoreGroupErrorEnum.DeleteError.getCode(), StoreGroupErrorEnum.DeleteError.getMessage());
        }
        TabStoreGroup tabStoreGroup = new TabStoreGroup();
        tabStoreGroup.setStoreGroupId(storeGroupId);
        tabStoreGroup.setStatus(0);
        return tabStoreGroupMapper.updateByPrimaryKeySelective(tabStoreGroup);
    }

    @Override
    public int insertDefaultStoreGroup(int enterpriseId) {
        StoreGroupDTO noStoreGroup = new StoreGroupDTO();
        noStoreGroup.setEnterpriseId(enterpriseId);
        noStoreGroup.setGroupLevel(StoreGroupConstant.ALL_STORE_LEVEL);
        noStoreGroup.setStoreGroupName("所有门店");
        noStoreGroup.setIsDefault(StoreGroupConstant.IS_DEFAULT);
        return save(noStoreGroup);
    }

    @Override
    public ServiceResponse<Integer> dragSort(Integer storeGroupId, Integer setSortValue) {
        TabStoreGroup tabStoreGroup = storeGroupService.getStoreGroupById(storeGroupId);
        if (tabStoreGroup == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "主键错误，查询不到数据");
        }
        if (setSortValue == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "拖拽排序值不能为空过");
        }
        storeGroupService.dragSort(tabStoreGroup, setSortValue);
        return ServiceResponse.success();
    }

    private int selectStoreGroupMaxSort(Integer enterpriseId, Integer groupLevel) {
        return tabStoreGroupMapper.selectStoreGroupMaxSort(enterpriseId, groupLevel);
    }

    private String getStoreGroupChain(Integer storeGroupId) {
        Stack<Integer> groupIds = new Stack<>();
        getParentStoreGroupIds(storeGroupId, groupIds);
        //所有门店的父级ID是0
        groupIds.push(StoreGroupConstant.ALL_STORE_LEVEL);
        //首个字符为下划线_
        StringBuilder resultId = new StringBuilder("_");
        while (!groupIds.empty()) {
            resultId.append(groupIds.pop());
            resultId.append("_");
        }
        return resultId.toString();
    }

    private void getParentStoreGroupIds(int storeGroupId, Stack<Integer> groupIds) {
        TabStoreGroup tabStoreGroup = tabStoreGroupMapper.selectByPrimaryKey(storeGroupId);
        if (tabStoreGroup != null) {
            groupIds.push(tabStoreGroup.getStoreGroupId());
            if (tabStoreGroup.getParentStoreGroupId() != StoreGroupConstant.ALL_STORE_LEVEL) {
                getParentStoreGroupIds(tabStoreGroup.getParentStoreGroupId(), groupIds);
            }
        }
    }

    private void updateSort(int storeGroupId, int sort) {
        TabStoreGroup tabStoreGroup = new TabStoreGroup();
        tabStoreGroup.setStoreGroupId(storeGroupId);
        tabStoreGroup.setSort(sort);
        tabStoreGroupMapper.updateByPrimaryKeySelective(tabStoreGroup);
    }

    /**
     * 是否需要转移门店，新增门店，判断上级门店是否是子节点，并且是否有门店
     * @param storeGroupDTO
     * @return
     */
    private boolean isNeedTransferStore(StoreGroupDTO storeGroupDTO) {
        int count = tabStoreGroupMapper.countByParentId(storeGroupDTO.getParentStoreGroupId());
        if (count < 1 && isContainStore(storeGroupDTO.getStoreGroupId())) {
            return true;
        } else {
            return false;
        }
    }

    private boolean isContainStore(int storeGroupId) {
        Integer result = storeService.countByGroupId(storeGroupId);
        if (result == null || result < 1) {
            return false;
        }
        return true;
    }

    private List<TabStoreGroup> listStoreGroupAndChildren(int storeGroupId, int enterpriseId) {
        TabStoreGroup param = new TabStoreGroup();
        param.setStoreGroupId(storeGroupId);
        param.setEnterpriseId(enterpriseId);
        List<TabStoreGroup> tabStoreGroupList = tabStoreGroupMapper.listStoreGroup(param);
        return tabStoreGroupList;
    }

    private void listParentStoreGroup(int parentId, Stack<TabStoreGroup> parentStoreGroupList) {
        TabStoreGroup tabStoreGroup = tabStoreGroupMapper.selectByPrimaryKey(parentId);
        if (tabStoreGroup != null) {
            parentStoreGroupList.push(tabStoreGroup);
            if (tabStoreGroup.getParentStoreGroupId() != StoreGroupConstant.ALL_STORE_LEVEL) {
                listParentStoreGroup(tabStoreGroup.getParentStoreGroupId(), parentStoreGroupList);
            }
        }
    }

    private List<TabStoreGroup> listStoreGroupByLevelAndName(int groupLevel, int enterpriseId, String storeGroupName) {
        TabStoreGroup param = new TabStoreGroup();
        param.setGroupLevel(groupLevel);
        param.setEnterpriseId(enterpriseId);
        param.setStoreGroupName(storeGroupName);
        List<TabStoreGroup> tabStoreGroupList = tabStoreGroupMapper.listStoreGroup(param);
        return tabStoreGroupList;
    }

    /**
     * 如果是默认门店，不可编辑
     * @param storeGroupId
     */
    private TabStoreGroup validIsDefault(int storeGroupId) {
        TabStoreGroup storeGroup = tabStoreGroupMapper.selectByPrimaryKey(storeGroupId);
        if (storeGroup == null) {
            throw new StoreGroupException(StoreGroupErrorEnum.StoreGroupNotExist.getCode(), StoreGroupErrorEnum.StoreGroupNotExist.getMessage());
        }
        boolean isDefault = storeGroup.getIsDefault() == StoreGroupConstant.IS_DEFAULT;
        if (isDefault) {
            throw new StoreGroupException(StoreGroupErrorEnum.EditDefaultStoreGroupError.getCode(), StoreGroupErrorEnum.EditDefaultStoreGroupError.getMessage());
        }
        return storeGroup;
    }
}
